const Clutter = imports.gi.Clutter;
const GLib = imports.gi.GLib;
const Lang = imports.lang;
const Main = imports.ui.main;
const Mainloop = imports.mainloop;
const Me = imports.misc.extensionUtils.getCurrentExtension();
const MessageTray = imports.ui.messageTray;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const St = imports.gi.St;
const Util = Me.imports.util;


const Indicator = new Lang.Class({
    Name: 'ReminderIndicator',
    Extends: PanelMenu.Button,
    _init: function () {
        this.parent(St.Align.START);
        this.settings = Util.getSettings();
        
        this.meter = new St.DrawingArea({reactive: false, width: 18, height: 18});
        this.meter.connect('repaint', Lang.bind(this, this.drawMeter));
        this.settings.connect('changed::enabled', 
            Lang.bind(this, function () { this.meter.queue_repaint(); }));
        this.actor.add_actor(this.meter);
        
        this.startTimer();
        

        this.buildMenu();
    },
    drawMeter: function (){
        let [width, height] = this.meter.get_surface_size();
        let enabled = this.settings.get_boolean('enabled');
        
        let cr = this.meter.get_context();
        let [res, c] = Clutter.Color.from_string('#ccc');
        let xc = width / 2;
        let yc = height / 2;
        let scale = Math.min(xc, yc) / 2;
        let r = scale * 1.5;

        let pct = (this.elapsed / 60) / this.settings.get_int('minutes');
        if (!enabled)
            [res, c] = Clutter.Color.from_string('#666');
        else if (pct >= 1)
            [res, c] = Clutter.Color.from_string('#c22');
        else if (pct >= .8)
            [res, c] = Clutter.Color.from_string('#855');
        else if (pct >= .9)
            [res, c] = Clutter.Color.from_string('#a33');
        Clutter.cairo_set_source_color(cr, c);

        cr.translate(xc, yc);
        cr.scale(.6, .6);

        cr.arc(0, -r * 1.6, r * .5, 0, 2 * Math.PI);
        cr.fill();

        cr.scale(1.5, 1.5);
        cr.translate(-r, -r);
        cr.moveTo(5.214844, 7.441406);
        cr.curveTo(5.214844, 7.441406, 6.328125, 8.558594, 7.441406, 7.441406);
        cr.curveTo(8.558594, 6.328125, 7.441406, 5.214844, 7.441406, 5.214844);
        cr.curveTo(7.441406, 5.214844, 1.3125, -0.917969, 0.199219, 0.199219);
        cr.curveTo(-0.917969, 1.3125, 5.214844, 7.441406, 5.214844, 7.441406);
        cr.closePath();
        cr.fill();

        let start = -1, end = Math.PI * 1.1;
        cr.translate(r, r);
        cr.arc(0, 0, r * 1.1, start, end);
        cr.stroke();
        
        if (pct < 1 && enabled)
            [res, c] = Clutter.Color.from_string('#080');
        pct = Math.min(end, pct * (end - start) + start);
        Clutter.cairo_set_source_color(cr, c);
        cr.arc(0, 0, r * 1.1, Math.max(pct - .5, start), pct);
        cr.stroke();
    },
    buildMenu: function () {
        let toggle = new PopupMenu.PopupSwitchMenuItem("", this.settings.get_boolean('enabled'));
        let message = 'Remind every %s minutes';
        let minutes = this.settings.get_int('minutes');
        
        toggle.label.set_text(message.format(minutes));
        toggle.connect('toggled', Lang.bind(this, function(){
            this.settings.set_boolean('enabled', toggle.state);
            if(toggle.state)
                this.startTimer();
        }));
        this.menu.addMenuItem(toggle);
        
        let slider = new PopupMenu.PopupSliderMenuItem(minutes / 59);
        slider.connect('value-changed', Lang.bind(this, function (){
            let val = Math.ceil(slider.value * 59) + 1;
            toggle.label.set_text(message.format(val));
            this.settings.set_int('minutes', val);
        }));
        this.menu.addMenuItem(slider);
        
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());

        let w = new PopupMenu.PopupMenuItem(_("Restart Timer"));
        w.connect('activate', Lang.bind(this, function(){ this.startTimer() }));
        this.menu.addMenuItem(w);
    },
    refreshTimer: function (timerId, initialMinutes){
      if(this.timerId != timerId || !this.settings.get_boolean('enabled'))
        return false;
      if(this.settings.get_int('minutes') < initialMinutes){
        // Pref must have been changed.  Just start over.
        this.startTimer();
        return false;
      }
      try{
        let minutes =  this.settings.get_int('minutes');
        let idleSeconds = GLib.spawn_command_line_sync("xprintidle")[1] / 1000;
        let adj = ((idleSeconds / 30) > .8) ? -Math.max(idleSeconds, 30) : 30;
        this.elapsed = Math.max(0, this.elapsed + adj);
        global.log('I:' + idleSeconds + ', A:' + adj + ', E:' + this.elapsed);
        Mainloop.timeout_add_seconds(30, 
            Lang.bind(this, this.refreshTimer, timerId, initialMinutes));
        this.meter.queue_repaint();
        if ((this.elapsed / 60) >= minutes) {
            this.timerFinished();
		} else if (this.source) {
			this.source.destroy();
			this.source = null;
		}
      }catch(e){
          global.log("error: " + e.toString());
      }
	  return false;
    },
    timerFinished: function () {
		let message = this.settings.get_string('message');
		if (message) {
			if (!this.source)
				this.source = new MessageTray.Source("Break Timer", '');
			if (!Main.messageTray.contains(this.source))
				Main.messageTray.add(this.source);
			if (!this.notification)
				this.notification = new MessageTray.Notification(this.source, "Break Reminder", message);
			this.source.notify(this.notification);
		}
    },
    startTimer: function () {
      this.timerId = new Object();
      this.elapsed = 0;
      this.refreshTimer(this.timerId, this.settings.get_int('minutes'));
    },
    destroy : function () {
        this.actor.destroy();
    }
});
